/*

Header file for send UDP spoofed packets (FAST version)
by Luigi Auriemma
e-mail: aluigi@autistici.org
web:    aluigi.org

VERSION 0.1.2


For use this header file, simply add #include "send_udp_fast.h" to
your source code.


This function need the following parameters:

- src     = 32 bit long number that identify the source IP (example:
            0x0100007f)
- dst     = 32 bit long number that identify the destination IP
- sport   = big endian source port (you must use htons) (example: port
            80 = 0x5000)
- dport   = big endian destination port (you must use htons)
- data    = data to insert in the packet
- datalen = size of the data to insert in the packet
- num     = how many packets send

NOTE:
- sport and dport must be in big endian
- if one of the port is equal to 0, it will be randomized
- ID and TTL of the IP header are randomized but are the same for all
  the packets
- if you need a more simple function for send spoofed UDP datagrams,
  you must use send_udp_full.h


Example:
send_udp_fast(
    inet_addr("127.0.0.1"),        //source
    IP_OF_"www.microsoft.com",    //destination (you need the
                                    //numeric IP of the destination
    htons(80),            //source port
    0,                //random destination port
    "Hello how are you?",        //data (NULL for empty packet)
    18,                //data length (0 for empty packet)
    3);                //number of packets


LINUX VERSION

*/

#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <string.h>
#include <time.h>



#define IPSZ        sizeof(struct iphdr)
#define UDPSZ        sizeof(struct udphdr)
#define PSEUDOSZ    sizeof(struct pseudohdr)
#define SIZE        (IPSZ + UDPSZ + datalen)
#define SENDUDPDELAY        0



void send_udp_fast(
    u_int    src,
    u_int    dst,
    u_short    sport,
    u_short    dport,
    u_char    *data,
    u_short    datalen,
    u_int    num);
u_short in_cksum(u_short *addr, int len);
void std_err(void);


struct pseudohdr {
    u_int32_t    saddr;
    u_int32_t    daddr;
    u_int8_t    zero;
    u_int8_t    protocol;
    u_int16_t    length;
} *pseudohdr;


/*
for more speed the ports must be given with htons (big endian) so the
function doesn't need to make this operation everytime

it supports the randomization of the source and dest port if equal to 0
*/


void send_udp_fast(
    u_int    src,
    u_int    dst,
    u_short    sport,
    u_short    dport,
    u_char    *data,
    u_short    datalen,
    u_int    num) {

    u_char    buff[SIZE],
            pseudobuff[PSEUDOSZ + UDPSZ + datalen],
            *datapck;
    struct    sockaddr_in     peer;
    struct    iphdr    *iphdr;
    struct    udphdr    *udphdr;
    int    sd,
        err;


    srand(time(NULL));

    if(!sport) sport = rand();
    if(!dport) dport = rand();

    peer.sin_addr.s_addr = dst;
    peer.sin_port        = dport;
    peer.sin_family      = AF_INET;

    iphdr     = (struct iphdr *)buff;
    udphdr    = (struct udphdr *)(buff + IPSZ);
    datapck   = (u_char *)(buff + IPSZ + UDPSZ);
    pseudohdr = (struct pseudohdr *)pseudobuff;


    /* build IP header */
    iphdr->ihl      = 5;
    iphdr->version  = 4;
    iphdr->tos      = 0x10;
    iphdr->tot_len  = SIZE;
    iphdr->id       = rand();
    iphdr->frag_off = 0;
    iphdr->ttl      = (rand() >> 24) + 120;
    iphdr->protocol = IPPROTO_UDP;
    iphdr->check    = 0;
    iphdr->saddr    = src;
    iphdr->daddr    = dst;

    /* build UDP header */
    udphdr->source = sport;
    udphdr->dest   = dport;
    udphdr->check  = 0;
    udphdr->len    = htons(UDPSZ + datalen);

    /* build data */
    memcpy(datapck, data, datalen);

    /* build pseudo header for calculate checksum (copy UDP header and data in it) */
    memcpy(pseudobuff + PSEUDOSZ, udphdr, UDPSZ + datalen);

    pseudohdr->saddr    = iphdr->saddr;
    pseudohdr->daddr    = iphdr->daddr;
    pseudohdr->zero     = 0;
    pseudohdr->protocol = IPPROTO_UDP;
    pseudohdr->length   = udphdr->len;

    udphdr->check = in_cksum((u_short *)pseudobuff, PSEUDOSZ + UDPSZ + datalen);


    /* send all */
    sd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
    if(sd < 0) std_err();

    for(; num > 0; num--) {
        err = sendto(sd, buff, SIZE, 0, (struct sockaddr *)&peer, sizeof(peer));
        if(err < 0) std_err();
        usleep(SENDUDPDELAY);
    }
    close(sd);
}



u_short in_cksum(u_short *addr, int len) {
        int    sum = 0;
        u_short    answer = 0;
        register    u_short *w = addr;
        register int    nleft = len;

        while(nleft > 1)  {
                sum += *w++;
                nleft -= 2;
        }
        if(nleft == 1) {
                *(u_char *)(&answer) = *(u_char *)w ;
                sum += answer;
        }
        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        answer = ~sum;
        return(answer);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}

